# Code for the BCSS tutorial on using JAGS for wildlife data analysis
# https://bcss.org.my/tut/bayes-with-jags-a-tutorial-for-wildlife-researchers/abundance-from-capture-recapture-data/basic-spatial-capture-recapture-models/
# Author: Mike Meredith
# License: GPL 3
# ----------------------------------------------------------------------------

library(jagsUI)
library(mcmcOutput)

# Get the data from the 'secr' package
# ------------------------------------
library(secr)
?stoatDNA
data(stoatDNA)
str(stoatCH)
plot(stoatCH, rad=50, tracks=TRUE)

# Extract the detector locations
traps <- traps(stoatCH)
str(traps)
# Convert to a matrix and metres to km
Detlocs <- as.matrix(traps) / 1000
( nDetlocs <- nrow(Detlocs) )

# Decide on state space
# ---------------------
# allow a buffer around the detector array
buffer <- 1  # kilometres
# outer edges of the state space
( xmin <- min(Detlocs[,1]) - buffer )
( xmax <- max(Detlocs[,1]) + buffer )
( ymin <- min(Detlocs[,2]) - buffer )
( ymax <- max(Detlocs[,2]) + buffer )

# Plot it:
plot(Detlocs, xlim=c(xmin, xmax), ylim=c(ymin, ymax),
    bty='n', pch=3, col='red')
rect(xmin, ymin, xmax, ymax, border='red', lwd=3)

# Reformat the data as a matrix
# -----------------------------
# Aggregate to an animals x detectors matrix with number of occasions detected
table(stoatCH)  # 0/1
dim(stoatCH)    # animals x occasions x detectors
y <- apply(stoatCH, c(1,3), sum)
dim(y)
table(y)
sum(y) == sum(stoatCH)
# All the detectors were active for all 7 days
nOcc <- 7

# Augment and bundle the data
# ---------------------------
( nCaps <- nrow(y) )
nAug <- 200  # number of rows to add
yAug <- rbind(y, matrix(0, nAug, nDetlocs))
( M <- nrow(yAug) )

# Bundle data for JAGS
jagsData <- list(y = yAug, M = M, nOcc = nOcc,
    w = c(rep(1, nCaps), rep(NA, M-nCaps)),
    Detlocs = Detlocs, nDetlocs = nDetlocs,
    xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax)
str(jagsData)
# List of 10
#  $ y       : num [1:220, 1:94] 0 0 0 0 0 0 0 0 0 0 ...
# $ M       : int 220
# $ nOcc    : num 7
# $ w       : num [1:220] 1 1 1 1 1 1 1 1 1 1 ...
# $ Detlocs : num [1:94, 1:2] -1.5 -1.5 -1.5 -1.5 -1.5 -1.5 -1.5 -1.5 -1.5 -1.5 ...
# $ nDetlocs: int 94
# $ xmin    : num -2.5
# $ xmax    : num 2.5
# $ ymin    : num -2.5
# $ ymax    : num 2.5

# Run the model
# -------------
wanted <- c("p0", "sigma", "omega", "N", "AC", "w")

set.seed(2)  # to get the same values as in the tutorial
out <- jags(jagsData, NULL, wanted, "SCRrect.jags", DIC=FALSE,
    n.chains=3, n.adapt=1000, n.iter=10000, parallel=TRUE)
  # takes 45 mins.
# save(out, file="stoats_jagsUI.RData")

# Check the output
# ----------------
library(mcmcOutput)
( mco <- mcmcOutput(out) )
# Object of class 'mcmcOutput'; approx. size 159.41 MB 
# MCMC values from jagsUI object ‘out’ 
# Model definition file: SCRrectangle.jags 
# Model run date: 2020-05-23 11:02:11 
# MCMC chain generation took 42.83 mins 
# Adaptation was sufficient.
# The output has 3 chains each with 10000 draws.
# It has 6 parameters with 664 nodes monitored:
#       nodes
# p0        1
# sigma     1
# omega     1
# N         1
# AC      440
# w       220

diagPlot(mco, params=1:4)
plot(mco, 1:4)
View(summary(mco))
# MCMC values from jagsUI object ‘out’ 
# The object has 664 nodes with 10000 draws for each of 3 chains.
# l95 and u95 are the limits of a 95% Highest Density Credible Interval.
# Rhat is the estimated potential scale reduction factor:
#         largest is 1.00; NONE are greater than 1.10; 20 (3%) are NA.
# MCEpc is the Monte Carlo standard error as a percentage of the posterior SD:
#         largest is 3.0%; NONE are greater than 5%; 20 (3%) are NA.


# check that N << M
max(mco$N)
M

# Estimate density
# ----------------
( area <- (xmax - xmin) * (ymax - ymin) ) # km2
D <- mco$N / area
postPlot(D, xlab="Density per sq km", showCurve=TRUE)
mean(D)
HDInterval::hdi(D)

# Estimate home range radius
# --------------------------
HRR <- mco$sigma * 2.45
postPlot(HRR, xlab="home range radius, km")

# Plot activity centres
# ---------------------
# reduce the number of iterations (30k is too many to plot)
mcox <- window(mco, thin=10)
# get AC coordinates, remove phantoms
ACall <- mcox$AC  #includes phantoms
w <- mcox$w
w[w==0] <- NA
ACs <- sweep(ACall, 1:2, w, "*")
str(ACs)
# num [1:3000, 1:220, 1:2] -1.71 -1.15 -1.74 -0.999 -1.48 ...
mean(is.na(ACs))  # phantoms have NA
# 0.6978864
post.means <- apply(ACs, 2:3, mean, na.rm=TRUE)
str(post.means)
# num [1:220, 1:2] -1.574 -1.543 -0.76 1.262 0.833 ...

# One big plot
par(mar=rep(1,4))
plot(Detlocs, xlim=c(xmin, xmax), ylim=c(ymin, ymax), pch=3, col='red',
        axes=FALSE)
for(i in 1:24)
  points(ACs[, i, ], pch='.', col=i)
plotrix::boxed.labels(x=post.means[1:20, 1], y=post.means[1:20, 2],
      labels=1:20)

# 4 smaller plots
par(mfrow=c(2,2), mar=rep(1,4))
toplot <- c(2,3,5,7,19,21,4,6,8,9,17,22,1,11,12,14,15,23,10,13,16,18,20,24)
colours <- rep(c(2:6,1), 4)
for(i in 1:24) {
  if(i %% 6 == 1)
    plot(Detlocs, xlim=c(xmin, xmax), ylim=c(ymin, ymax), pch=3, col='red',
        axes=FALSE)
  points(ACs[, toplot[i], ], pch='.', col=colours[i])
  if(i %% 6 !=0)
    plotrix::boxed.labels(x=post.means[toplot[i], 1], y=post.means[toplot[i], 2],
      labels=toplot[i])
}

# Density estimate based on smaller 'core' area
# ---------------------------------------------
# allow a buffer of only 250m around the detector array
corebuf <- 0.25  # kilometres
# outer edges of the state space
cxmin <- min(Detlocs[,1]) - corebuf
cxmax <- max(Detlocs[,1]) + corebuf
cymin <- min(Detlocs[,2]) - corebuf
cymax <- max(Detlocs[,2]) + corebuf

# plot it
par(mar=rep(1,4))
plot(Detlocs, xlim=c(xmin, xmax), ylim=c(ymin, ymax), pch=3, col='red',
        axes=FALSE)
for(i in 1:24)
  points(ACs[, i, ], pch='.', col=i)
rect(cxmin, cymin, cxmax, cymax, lwd=2, border='blue')

NinCore <- numeric(dim(ACs)[1])
for(i in seq_along(NinCore)) {
  draw <- ACs[i,,]
  inCore <- draw[,1] > cxmin & draw[,1] < cxmax & 
      draw[,2] > cymin & draw[,2] < cymax 
  NinCore[i] <- sum(inCore, na.rm=TRUE)
}
postPlot(NinCore, xlab="Number in core")

# convert to density
( Acore <- (cxmax - cxmin) * (cymax - cymin) ) # km2
# 12.25
Dcore <- NinCore / Acore
postPlot(Dcore, xlab="Density per sq km", showCurve=TRUE)

# Do plots side by side
par(mfrow=1:2)
postPlot(NinCore, xlab="Number in core")
postPlot(Dcore, xlab="Density per sq km", showCurve=TRUE)
