# Code for the BCSS tutorial on using JAGS for wildlife data analysis
# https://bcss.org.my/tut/bayes-with-jags-a-tutorial-for-wildlife-researchers/abundance-from-capture-recapture-data/basic-spatial-capture-recapture-models/
# Author: Mike Meredith
# License: GPL 3
# ----------------------------------------------------------------------------

# Analysis with 'nimble' package instead of JAGS

library(nimble)
library(mcmcOutput)

# Get the data from the 'secr' package
# ------------------------------------
library(secr)
?stoatDNA
data(stoatDNA)
str(stoatCH)
plot(stoatCH, rad=50, tracks=TRUE)

# Extract the detector locations
traps <- traps(stoatCH)
str(traps)
# Convert to a matrix and metres to km
Detlocs <- as.matrix(traps) / 1000
( nDetlocs <- nrow(Detlocs) )

# Decide on state space
# ---------------------
# allow a buffer around the detector array
buffer <- 1  # kilometres
# outer edges of the state space
( xmin <- min(Detlocs[,1]) - buffer )
( xmax <- max(Detlocs[,1]) + buffer )
( ymin <- min(Detlocs[,2]) - buffer )
( ymax <- max(Detlocs[,2]) + buffer )

# Plot it:
plot(Detlocs, xlim=c(xmin, xmax), ylim=c(ymin, ymax),
    bty='n', pch=3, col='red')
rect(xmin, ymin, xmax, ymax, border='red', lwd=3)

# Reformat the data as a matrix
# -----------------------------
# Aggregate to an animals x detectors matrix with number of occasions detected
table(stoatCH)  # 0/1
dim(stoatCH)    # animals x occasions x detectors
y <- apply(stoatCH, c(1,3), sum)
dim(y)
table(y)
sum(y) == sum(stoatCH)
# All the detectors were active for all 7 days
nOcc <- 7

# Write the model for NIMBLE
# --------------------------
SCRrect.code <- nimbleCode({
  # Likelihood
  for(i in 1:M){                 # Loop over all M individuals
    w[i] ~ dbern(omega)          # w = 1 if animal is real/present
    AC[i, 1] ~ dunif(xmin, xmax) # x-coord of activity centre
    AC[i, 2] ~ dunif(ymin, ymax) # y coord of activity centre

    for(j in 1:nDetlocs){        # Loop over all detectors
      d2[i,j] <- (AC[i,1] - Detlocs[j,1])^2 + (AC[i,2] - Detlocs[j,2])^2              # distance^2
      p[i,j] <- p0 * exp(- d2[i,j]/(2*sigma^2)) # Detection prob
      y[i,j] ~ dbin(p[i,j] * w[i], nOcc)        # The observed data
    }
  }

  # Priors
  p0 ~ dbeta(1, 1)          # Baseline detection probability
  sigma ~ dunif(0, 3)       # Half-normal scale
  omega ~ dbeta(1, 1)       # Data augmentation parameter

  # Derived quantities
  N <- sum(w[1:M])          # Population size in state-space (=area)
})

# Augment and bundle the data
# ---------------------------
( nCaps <- nrow(y) )
nAug <- 200  # number of rows to add
yAug <- rbind(y, matrix(0, nAug, nDetlocs))
( M <- nrow(yAug) )

# Bundle data for NIMBLE
nimData <- list(y = yAug, w = c(rep(1, nCaps), rep(NA, M-nCaps)))
nimConst <- list(M = M, nOcc = nOcc, Detlocs = Detlocs, nDetlocs = nDetlocs,
    xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax)
str(nimData)
str(nimConst)

# Run the model
# -------------
wanted <- c("p0", "sigma", "omega", "N", "AC", "w")

system.time(
out1 <- nimbleMCMC(SCRrect.code, nimConst, nimData, monitors=wanted,
    nchains=3, niter=11e3, nburnin=1e3, samplesAsCodaMCMC=TRUE) ) # 15 mins
diagPlot(out1, wanted[1:4])
# save(out, file="stoats_NIMBLE.RData")

# Check the output
# ----------------
( mco1 <- mcmcOutput(out1) )

plot(mco1, wanted[1:4])
View(summary(mco1))

# check that N << M
max(mco1$N)
M

# Estimate density
# ----------------
( area <- (xmax - xmin) * (ymax - ymin) ) # km2
D <- mco1$N / area
postPlot(D, xlab="Density per sq km", showCurve=TRUE)
mean(D)
HDInterval::hdi(D)

# Estimate home range radius
# --------------------------
HRR <- mco1$sigma * 2.45
postPlot(HRR, xlab="home range radius, km")

# Plot activity centres
# ---------------------
# reduce the number of iterations (30k is too many to plot)
mcox <- window(mco1, thin=10)
# get AC coordinates, remove phantoms
ACall <- mcox$AC  #includes phantoms
w <- mcox$w
w[w==0] <- NA
ACs <- sweep(ACall, 1:2, w, "*")
str(ACs)
# num [1:3000, 1:220, 1:2] -1.71 -1.15 -1.74 -0.999 -1.48 ...
mean(is.na(ACs))  # phantoms have NA
# 0.6978864
post.means <- apply(ACs, 2:3, mean, na.rm=TRUE)
str(post.means)
# num [1:220, 1:2] -1.574 -1.543 -0.76 1.262 0.833 ...

# One big plot
par(mar=rep(1,4))
plot(Detlocs, xlim=c(xmin, xmax), ylim=c(ymin, ymax), pch=3, col='red',
        axes=FALSE)
for(i in 1:24)
  points(ACs[, i, ], pch='.', col=i)
plotrix::boxed.labels(x=post.means[1:20, 1], y=post.means[1:20, 2],
      labels=1:20)

# 4 smaller plots
op <- par(mfrow=c(2,2), mar=rep(1,4))
toplot <- c(2,3,5,7,19,21,4,6,8,9,17,22,1,11,12,14,15,23,10,13,16,18,20,24)
colours <- rep(c(2:6,1), 4)
for(i in 1:24) {
  if(i %% 6 == 1)
    plot(Detlocs, xlim=c(xmin, xmax), ylim=c(ymin, ymax), pch=3, col='red',
        axes=FALSE)
  points(ACs[, toplot[i], ], pch='.', col=colours[i])
  if(i %% 6 !=0)
    plotrix::boxed.labels(x=post.means[toplot[i], 1], y=post.means[toplot[i], 2],
      labels=toplot[i])
}
par(op)

# Density estimate based on smaller 'core' area
# ---------------------------------------------
# allow a buffer of only 250m around the detector array
corebuf <- 0.25  # kilometres
# outer edges of the state space
cxmin <- min(Detlocs[,1]) - corebuf
cxmax <- max(Detlocs[,1]) + corebuf
cymin <- min(Detlocs[,2]) - corebuf
cymax <- max(Detlocs[,2]) + corebuf

# plot it
op <- par(mar=rep(1,4))
plot(Detlocs, xlim=c(xmin, xmax), ylim=c(ymin, ymax), pch=3, col='red',
        axes=FALSE)
for(i in 1:24)
  points(ACs[, i, ], pch='.', col=i)
rect(cxmin, cymin, cxmax, cymax, lwd=2, border='blue')
par(op)

NinCore <- numeric(dim(ACs)[1])
for(i in seq_along(NinCore)) {
  draw <- ACs[i,,]
  inCore <- draw[,1] > cxmin & draw[,1] < cxmax &
      draw[,2] > cymin & draw[,2] < cymax
  NinCore[i] <- sum(inCore, na.rm=TRUE)
}
postPlot(NinCore, xlab="Number in core")

# convert to density
( Acore <- (cxmax - cxmin) * (cymax - cymin) ) # km2
# 12.25
Dcore <- NinCore / Acore
postPlot(Dcore, xlab="Density per sq km", showCurve=TRUE)

# Do plots side by side
op <- par(mfrow=1:2)
postPlot(NinCore, xlab="Number in core")
postPlot(Dcore, xlab="Density per sq km", showCurve=TRUE)
par(op)
